package com.muyu.system.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.muyu.common.core.text.Convert;
import com.muyu.common.security.utils.SecurityUtils;
import com.muyu.common.system.domain.SysDept;
import com.muyu.common.system.domain.SysUser;
import com.muyu.system.domain.AsNoticeUser;
import com.muyu.system.domain.SysNotice;
import com.muyu.system.domain.SysPost;
import com.muyu.system.domain.req.AsNoticeUserReq;
import com.muyu.system.domain.req.SelectAsNoticeUserReq;
import com.muyu.system.domain.resp.AsNoticeUserResp;
import com.muyu.system.domain.resp.SysNoticeResp;
import com.muyu.system.mapper.SysNoticeMapper;
import com.muyu.system.service.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 公告 服务层实现
 *
 * @author muyu
 */
@Service
public class SysNoticeServiceImpl extends ServiceImpl<SysNoticeMapper, SysNotice> implements SysNoticeService {
    @Autowired
    private SysNoticeMapper noticeMapper;

    @Autowired
    private AsNoticeUserService asNoticeUserService;

    @Autowired
    private SysUserService userService;

    @Autowired
    private SysPostService postService;

    @Autowired
    private SysDeptService deptService;


    /**
     * 查询公告信息
     *
     * @param noticeId 公告ID
     * @return 公告信息
     */
    @Override
    public SysNotice selectNoticeById(Long noticeId) {
        return noticeMapper.selectNoticeById(noticeId);
    }

    /**
     * 查询公告列表
     *
     * @param notice 公告信息
     * @return 公告集合
     */
    @Override
    public List<SysNoticeResp> selectNoticeList(SysNotice notice) {
        List<SysNotice> sysNotices = noticeMapper.selectNoticeList(notice);
        List<SysNoticeResp> sysNoticeRespList = sysNotices.stream().map(sysNotice -> {
            //通知总数
            List<AsNoticeUser> allList = asNoticeUserService.list(new LambdaQueryWrapper<AsNoticeUser>() {{
                eq(AsNoticeUser::getNoticeId, sysNotice.getNoticeId());
            }});
            //已读数量
            long allCount = allList.size();
            long readCount = allList.stream().filter(user -> "Y".equals(user.getReadStates())).count();
            long unReadCount = allCount - readCount;

            SysNoticeResp sysNoticeResp = SysNoticeResp.saveBuilder(sysNotice);
            if (!allList.isEmpty()) {
                sysNoticeResp.setEndTime(allList.get(0).getEndTime());
            }
            sysNoticeResp.setNum(allCount);
            sysNoticeResp.setReadNum(readCount);
            sysNoticeResp.setUnReadNum(unReadCount);
            return sysNoticeResp;
        }).collect(Collectors.toList());
        return sysNoticeRespList;
    }

    /**
     * 新增公告
     *
     * @param req 公告信息
     * @return 结果
     */
    @Override
    public int insertNotice(AsNoticeUserReq req) {
        SysNotice sysNotice = SysNotice.saveBuilder(req, SecurityUtils::getUsername);
        int i = noticeMapper.insertNotice(sysNotice);

        if (i > 0) {
            SysUser sysUser = userService.selectUserById(SecurityUtils.getUserId());
            List<SysPost> list = postService.selectPostListByUserId(SecurityUtils.getUserId()).stream().map(postId -> postService.selectPostById(postId)).toList();
            SysDept dept = deptService.selectDeptById(sysUser.getDeptId());
            StringBuffer buffer = new StringBuffer();
            list.forEach(sysPost -> buffer.append(sysPost.getPostName()));
            req.setCreateBy(sysUser.getUserName() + "(" + dept.getDeptName() + "/" + String.valueOf(buffer) + ")");
            List<Long> userCheckedList = req.getUserCheckedList();
            List<Long> finalUserCheckedList = userCheckedList;
            req.getSelectedOptions().forEach(deptId ->
                    finalUserCheckedList.addAll(userService.selectUserList(new SysUser() {{
                                setDeptId(deptId);
                            }}).stream().map(SysUser::getUserId).toList()
                    ));
            userCheckedList = userCheckedList.stream().distinct().toList();
            List<AsNoticeUser> asNoticeUserList = userCheckedList.stream().map(userId ->
                    AsNoticeUser.builder()
                            .endTime(req.getInDateTime())
                            .userId(userId)
                            .noticeId(sysNotice.getNoticeId())
                            .outStates("N")
                            .readStates("N")
                            .createBy(req.getCreateBy())
                            .createTime(new Date())
                            .build()).collect(Collectors.toList());

            asNoticeUserService.saveBatch(asNoticeUserList);
        }
        return i;
    }

    /**
     * 修改公告
     *
     * @param notice 公告信息
     * @return 结果
     */
    @Override
    public int updateNotice(SysNotice notice) {
        return noticeMapper.updateNotice(notice);
    }

    /**
     * 删除公告对象
     *
     * @param noticeId 公告ID
     * @return 结果
     */
    @Override
    public int deleteNoticeById(Long noticeId) {
        return noticeMapper.deleteNoticeById(noticeId);
    }

    /**
     * 批量删除公告信息
     *
     * @param noticeIds 需要删除的公告ID
     * @return 结果
     */
    @Override
    public int deleteNoticeByIds(Long[] noticeIds) {
        return noticeMapper.deleteNoticeByIds(noticeIds);
    }

    /**
     * 用户获取收到的通知和公告
     *
     * @return
     */
    @Override
    public List<AsNoticeUserResp> noticeByUserId(SelectAsNoticeUserReq req) {
        List<AsNoticeUser> list = asNoticeUserService.
                list(new LambdaQueryWrapper<AsNoticeUser>() {{
                         eq(AsNoticeUser::getUserId, SecurityUtils.getUserId());
                         ge(AsNoticeUser::getEndTime, new Date());
                         if (req.getReadStates() != null && !req.getReadStates().isEmpty())
                             eq(AsNoticeUser::getReadStates, req.getReadStates());
                     }}
                );
        List<AsNoticeUserResp> result = new ArrayList<>();
        list.forEach(noticeUser -> {
            SysNotice sysNotice = this.getOne(new LambdaQueryWrapper<SysNotice>() {{
                eq(SysNotice::getNoticeId, noticeUser.getNoticeId());
                if (req.getNoticeType() != null && !req.getNoticeType().isEmpty() &&!req.getNoticeType().equals("0"))
                    eq(SysNotice::getNoticeType, req.getNoticeType());
            }});
            if (sysNotice!=null){
                AsNoticeUserResp apply = AsNoticeUserResp.builder()
                        .asNoticeId(noticeUser.getId())
                        .endTime(noticeUser.getEndTime())
                        .createTime(noticeUser.getCreateTime())
                        .createBy(noticeUser.getCreateBy())
                        .noticeType(sysNotice.getNoticeType())
                        .readStates(noticeUser.getReadStates())
                        .sysNotice(sysNotice)
                        .build();
                result.add(apply);
            }
        });
        return result;
    }

    @Override
    public int updateAsNoticeUser(Long asNoticeId) {

        AsNoticeUser asNoticeUser = new AsNoticeUser() {{
            setReadStates("Y");
            setId(asNoticeId);
        }};
        return asNoticeUserService.updateAsNoticeUser(asNoticeUser);
    }
}
